from io import BytesIO

from PIL import Image as PilImage

PilImage.IGNORE_DECODING_ERRORS = True
PilImage.MAXBLOCK = 2 ** 25


class Image:
    @classmethod
    def create_from_buffer(cls, image_buffer):
        instance = cls()
        return instance.parse_image(image_buffer)

    def parse_image(self, image_buffer):
        tmp = BytesIO(image_buffer)
        try:
            img = PilImage.open(tmp)
            if hasattr(img, 'is_animated') and img.is_animated:
                return None
        except IOError:
            return None
        try:
            img.load()
        except IOError:
            pass
        img = img.convert('L')
        tmp.close()
        return img



from os.path import join, dirname
from remotecv.image import Image


def read_fixture(path):
    with open(join(dirname(__file__), 'fixtures', path)) as f:
        return f.read()


def create_image(path):
    return Image.create_from_buffer(read_fixture(path))



#!/usr/bin/python
# -*- coding: utf-8 -*-

import mock
from preggy import expect
from unittest import TestCase
from tests import read_fixture

from remotecv.pyres_tasks import DetectTask
from remotecv.utils import config


class DetectTaskTestCase(TestCase):

    def test_should_run_detector_task(self):
        store_mock = mock.Mock()
        config.loader = mock.Mock(load_sync=read_fixture)
        config.store = store_mock
        config.store.ResultStore = mock.Mock(return_value=store_mock)
        DetectTask.perform('all', 'multiple_faces_bw.jpg', 'test-key')
        call = store_mock.store.call_args[0]
        expect(call[0]).to_equal('test-key')
        expect(call[1]).to_be_greater_than(20)
        expect(call[1][0][0]).to_be_numeric()
        expect(call[1][0][1]).to_be_numeric()
        expect(call[1][0][2]).to_be_numeric()
        expect(call[1][0][3]).to_be_numeric()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from preggy import expect
from unittest import TestCase
from tests import create_image
from remotecv.detectors.profile_detector import ProfileDetector


class ProfileDetectorTestCase(TestCase):

    def test_should_detect_one_face(self):
        detection_result = ProfileDetector().detect(create_image('profile_face.jpg'))
        expect(detection_result).to_length(1)
        expect(detection_result[0][0]).to_be_numeric()
        expect(detection_result[0][1]).to_be_numeric()
        expect(detection_result[0][2]).to_be_numeric()
        expect(detection_result[0][3]).to_be_numeric()

    def test_should_not_detect(self):
        detection_result = ProfileDetector().detect(create_image('no_face.jpg'))
        expect(detection_result).to_length(0)



import pkgutil
import pkg_resources
import sys

from remotecv.utils import config

class ErrorHandler:
    def __init__(self, sentry_url):
        self.sentry_client = None

        if not sentry_url:
            return

        from raven import Client
        self.sentry_client = Client(sentry_url)
        self.extra_data = {
            'modules': self.get_modules()
        }
        self.install_handler()

    def get_modules(self):
        resolved = {}
        modules = [mod[1] for mod in tuple(pkgutil.iter_modules())]
        for module in modules:
            try:
                res_mod = pkg_resources.get_distribution(module)
                if res_mod is not None:
                    resolved[module] = res_mod.version
            except pkg_resources.DistributionNotFound:
                pass

        return resolved

    def install_handler(self):
        if not self.sentry_client:
            return
        old_hook = sys.excepthook
        def on_exception(type, value, traceback):
            try:
                self.sentry_client.captureException((type, value, traceback), data=self.extra_data)
            finally:
                old_hook(type, value, traceback)
        sys.excepthook = on_exception



#!/usr/bin/python
# -*- coding: utf-8 -*-

from preggy import expect
from unittest import TestCase
from tests import create_image

from remotecv.detectors.feature_detector import FeatureDetector


class FeatureDetectorTestCase(TestCase):
    def test_should_detect_multiple_points(self):
        detection_result = FeatureDetector().detect(create_image('no_face.jpg'))
        expect(detection_result).to_be_greater_than(4)

    def test_should_not_detect_points(self):
        detection_result = FeatureDetector().detect(create_image('white-block.png'))
        expect(detection_result).to_be_false()



from pyres import ResQ
from pyres.worker import Worker

from remotecv.utils import logger

class UniqueQueue(ResQ):
    def _escape_for_key(self, value):
        return value.replace(" ", "").replace("\n", "")

    def _create_unique_key(self, queue, key):
        return "resque:unique:queue:%s:%s" % (queue, self._escape_for_key(str(key)))

    def add_unique_key(self, queue, key):
        unique_key = self._create_unique_key(queue, key)
        if self.redis.get(unique_key) == "1":
            # Do nothing as this message is already enqueued
            return False

        self.redis.set(unique_key, "1")
        return True

    def del_unique_key(self, queue, key):
        unique_key = self._create_unique_key(queue, key)
        self.redis.delete(unique_key)

    def enqueue_unique_from_string(self, klass_as_string, queue, args=None, key=None):
        if not self.add_unique_key(queue, key):
            logger.debug('key %s already enqueued' % key)
            return

        if not args:
            args = []

        payload = {'class':klass_as_string, 'queue': queue, 'args':args + [key], 'key':key}
        self.push(queue, payload)
        logger.info("enqueued '%s' job on queue %s" % (klass_as_string, queue))
        if args:
            logger.debug("job arguments: %s" % str(args))
        else:
            logger.debug("no arguments passed in.")

class UniqueWorker(Worker):

    def __init__(self, queues=(), server="localhost:6379", password=None, timeout=None, after_fork=None):
        self.after_fork = after_fork
        super(UniqueWorker, self).__init__(queues, UniqueQueue(server=server), password, timeout)

    def after_fork(self, job):
        if self.after_fork and callable(self.after_fork):
            self.after_fork(job)

    def before_process(self, job):
        self.resq.del_unique_key(job._queue, job._payload['key'])
        return job



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/globocom/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

from remotecv.detectors import CascadeLoaderDetector
from remotecv.detectors.face_detector import FaceDetector
from remotecv.detectors.glasses_detector import GlassesDetector
from remotecv.detectors.profile_detector import ProfileDetector
from remotecv.detectors.feature_detector import FeatureDetector


class CompleteDetector(CascadeLoaderDetector):

    def detect(self, image):
        face_detector = FaceDetector()
        glasses_detector = GlassesDetector()
        profile_detector = ProfileDetector()
        feature_detector = FeatureDetector()

        face_points = face_detector.detect(image) or []
        glasses_points = glasses_detector.detect(image) or []
        profile_points = profile_detector.detect(image) or []
        feature_points = feature_detector.detect(image) or []

        return face_points + glasses_points + profile_points + feature_points



#!/usr/bin/python
# -*- coding: utf-8 -*-

from preggy import expect
from unittest import TestCase
from tests import create_image

from remotecv.detectors.glasses_detector import GlassesDetector


class GlassesDetectorTestCase(TestCase):
    def test_should_detect_glasses(self):
        detection_result = GlassesDetector().detect(create_image('glasses.jpg'))
        expect(detection_result).to_length(2)
        expect(detection_result[0][0]).to_be_numeric()
        expect(detection_result[0][1]).to_be_numeric()
        expect(detection_result[0][2]).to_be_numeric()
        expect(detection_result[0][3]).to_be_numeric()
        expect(detection_result[1][0]).to_be_numeric()
        expect(detection_result[1][1]).to_be_numeric()
        expect(detection_result[1][2]).to_be_numeric()
        expect(detection_result[1][3]).to_be_numeric()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from preggy import expect
from unittest import TestCase
from tests import create_image
from remotecv.detectors.complete_detector import CompleteDetector


class CompleteDetectorTestCase(TestCase):
    def test_should_detect_something(self):
        detection_result = CompleteDetector().detect(create_image('profile_face.jpg'))
        expect(detection_result).to_be_greater_than(20)
        expect(detection_result[0][0]).to_be_numeric()
        expect(detection_result[0][1]).to_be_numeric()
        expect(detection_result[0][2]).to_be_numeric()
        expect(detection_result[0][3]).to_be_numeric()



from unittest import TestCase
from preggy import expect
from tests import read_fixture
from nose.tools import raises

from remotecv.image_processor import ImageProcessor


class ImageProcessorTest(TestCase):

    @raises(AttributeError)
    def test_when_detector_unavailable(self):
        image_processor = ImageProcessor()
        image_processor.detect('feat', read_fixture('broken.jpg'))

    def test_when_image_is_huge(self):
        image_processor = ImageProcessor()
        detect = image_processor.detect('all', read_fixture('huge_image.jpg'))
        expect(detect).Not.to_be_empty()

    def test_when_not_animated_gif(self):
        image_processor = ImageProcessor()
        detect = image_processor.detect('face', read_fixture('one_face.gif'))
        expect(detect).Not.to_be_empty()

    def test_when_animated_gif(self):
        image_processor = ImageProcessor()
        detect = image_processor.detect('all', read_fixture('animated.gif'))
        expect(detect).to_be_empty()

    def test_feature_detection(self):
        image_processor = ImageProcessor()
        detect = image_processor.detect('feature', read_fixture('broken.jpg'))
        expect(detect).Not.to_be_empty()

    def test_should_be_empty_when_invalid_image(self):
        image_processor = ImageProcessor()
        detect = image_processor.detect('all', 'asdas')
        expect(detect).to_be_empty()

    def test_should_ignore_gif(self):
        image_processor = ImageProcessor()
        detect = image_processor.detect('all', 'asdas')
        expect(detect).to_be_empty()



from remotecv.image_processor import ImageProcessor
from remotecv.utils import config, logger


class DetectTask:
    queue = "Detect"
    processor = ImageProcessor()

    @classmethod
    def perform(clz, detection_type, image_path, key):
        logger.info('Detecting %s for %s' % (detection_type, image_path))
        image_data = config.loader.load_sync(image_path)
        points = clz.processor.detect(detection_type, image_data)
        result_store = config.store.ResultStore(config)
        result_store.store(key, points)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# remote cv service
# https://github.com/globocom/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

import sys
import argparse
import logging

from redis import Redis
from pyres import ResQ
from itty import run_itty
from resweb import server as resweb_server

def main(params=None):
    if params is None:
        params = sys.argv[1:]
    parser = argparse.ArgumentParser(description='Runs pyres web console.')

    conn_group = parser.add_argument_group('Connection arguments')
    conn_group.add_argument('--host', default='localhost', help='Binding host')
    conn_group.add_argument('--port', default=8080, type=int, help='Binding port')

    conn_group = parser.add_argument_group('Redis arguments')
    conn_group.add_argument('--redis-host', default='localhost', help='Redis host')
    conn_group.add_argument('--redis-port', default=6379, type=int, help='Redis port')
    conn_group.add_argument('--redis-database', default=0, type=int, help='Redis database')
    conn_group.add_argument('--redis-password', default=None, help='Redis password')

    other_group = parser.add_argument_group('Other arguments')
    other_group.add_argument('-l', '--level', default='debug', help='Logging level')

    arguments = parser.parse_args(params)
    logging.basicConfig(level=getattr(logging, arguments.level.upper()))

    redis = Redis(host=arguments.redis_host, port=arguments.redis_port, db=arguments.redis_database, password=arguments.redis_password)
    resweb_server.HOST = ResQ(redis)
    run_itty(host=arguments.host, port=arguments.port, server='wsgiref')

if __name__ == '__main__':
    main()



from unittest import TestCase
from preggy import expect
from tests import create_image


class ImageTest(TestCase):

    def test_should_read_a_psd(self):
        im = create_image('why_not_a.psd')
        expect(im).Not.to_be_null()
        expect(im.size).to_equal((620, 413))

    def test_when_image_is_broken(self):
        im = create_image('broken.jpg')
        expect(im).Not.to_be_null()
        expect(im.size).to_equal((600, 769))

    def test_when_image_is_pallete(self):
        im = create_image('pallete.png')
        expect(im).Not.to_be_null()
        expect(im.size).to_equal((3317, 2083))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from preggy import expect
from unittest import TestCase
from tests import create_image
from remotecv.detectors.face_detector import FaceDetector


class FaceDetectorTestCase(TestCase):

    def test_should_detect_one_face(self):
        detection_result = FaceDetector().detect(create_image('one_face.jpg'))
        expect(detection_result).to_length(1)
        expect(detection_result[0][0]).to_be_numeric()
        expect(detection_result[0][1]).to_be_numeric()
        expect(detection_result[0][2]).to_be_numeric()
        expect(detection_result[0][3]).to_be_numeric()

    def test_should_not_detect(self):
        detection_result = FaceDetector().detect(create_image('no_face.jpg'))
        expect(detection_result).to_be_empty()

    def test_should_run_on_grayscale_images(self):
        detection_result = FaceDetector().detect(create_image('multiple_faces_bw.jpg'))
        expect(detection_result).to_be_greater_than(4)
        expect(detection_result[0][0]).to_be_numeric()
        expect(detection_result[0][1]).to_be_numeric()
        expect(detection_result[0][2]).to_be_numeric()
        expect(detection_result[0][3]).to_be_numeric()

    def test_should_run_on_cmyk_images(self):
        detection_result = FaceDetector().detect(create_image('one_face_cmyk.jpg'))
        expect(detection_result[0][0]).to_be_numeric()
        expect(detection_result[0][1]).to_be_numeric()
        expect(detection_result[0][2]).to_be_numeric()
        expect(detection_result[0][3]).to_be_numeric()

    def test_should_run_on_images_with_alpha(self):
        detection_result = FaceDetector().detect(create_image('one_face.png'))
        expect(detection_result[0][0]).to_be_numeric()
        expect(detection_result[0][1]).to_be_numeric()
        expect(detection_result[0][2]).to_be_numeric()
        expect(detection_result[0][3]).to_be_numeric()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/globocom/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

from setuptools import setup, find_packages
from remotecv import __version__

tests_require = [
    "nose",
    "preggy",
    "coverage",
    "numpy",
    "yanc",
    "nose-focus",
    "mock",
]

setup(
    name='remotecv',
    version=__version__,
    description="remotecv is an OpenCV worker for facial and feature recognition",
    long_description="""
remotecv is an OpenCV worker for facial and feature recognition
""",
    keywords='imaging face detection feature opencv',
    author='globo.com',
    author_email='timehome@corp.globo.com',
    url='https://github.com/globocom/remotecv/wiki',
    license='MIT',
    classifiers=['Development Status :: 4 - Beta',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: MIT License',
                 'Natural Language :: English',
                 'Operating System :: MacOS',
                 'Operating System :: POSIX :: Linux',
                 'Programming Language :: Python :: 2.6',
                 'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
                 'Topic :: Multimedia :: Graphics :: Presentation'],
    packages=find_packages(),
    package_data={
        '': ['*.xml'],
    },

    extras_require={
        'tests': tests_require,
    },

    install_requires=[
        "argparse>=1.2.1,<1.3.0",
        "pyres>=1.2,<1.3",
        "Pillow>=2.3.0,<3.0.0",
    ],

    entry_points={
        'console_scripts': [
            'remotecv = remotecv.worker:main',
            'remotecv-web = remotecv.web:main'
        ],
    }
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/globocom/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

from os.path import join, dirname, abspath

import cv2
import numpy as np


class BaseDetector(object):
    def get_np_img(self, image):
        return np.array(image)

    def detect(self, context):
        raise NotImplementedError()


class CascadeLoaderDetector(BaseDetector):

    def load_cascade_file(self, module_path, cascade_file_path):
        cascade_file = join(abspath(dirname(module_path)), cascade_file_path)
        self.__class__.cascade = cv2.CascadeClassifier(cascade_file)

    def get_min_size_for(self, size):
        ratio = int(min(size[0], size[1]) / 15)
        ratio = max(20, ratio)
        return (ratio, ratio)

    def get_features(self, image):
        img = self.get_np_img(image)

        faces = self.__class__.cascade.detectMultiScale(
            img,
            1.2,
            4,
            minSize=self.get_min_size_for(image.size)
        )
        faces_scaled = []

        for (x, y, w, h) in faces:
            faces_scaled.append(
                (
                    (
                        x.item(),
                        y.item(),
                        w.item(),
                        h.item()
                    ), 0)
            )

        return faces_scaled

    def detect(self, image):
        features = self.get_features(image)

        if features:
            points = [[left, top, width, height] for (left, top, width, height), neighbors in features]
        else:
            points = []

        return points



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/globocom/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

import cv2

from remotecv.detectors import BaseDetector


class FeatureDetector(BaseDetector):
    def detect(self, image):
        points = cv2.goodFeaturesToTrack(
            self.get_np_img(image),
            maxCorners=20,
            qualityLevel=0.04,
            minDistance=1.0,
            useHarrisDetector=False,
        )

        if points is not None and len(points) > 0:
            return [[point[0][0].item(), point[0][1].item(), 1, 1] for point in points]

        return None
